package nl.iobyte.themepark.menu;

import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.attraction.Region;
import nl.iobyte.themepark.api.attraction.component.Status;
import nl.iobyte.themepark.api.attraction.component.Type;
import nl.iobyte.themepark.api.attraction.manager.StatusManager;
import nl.iobyte.themepark.api.utils.LocationSerializer;
import nl.iobyte.themepark.config.YamlConfig;
import nl.iobyte.themepark.database.DB;
import nl.iobyte.themepark.database.DBManager;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class AttractionLoader {

    private static HashMap<String, YamlConfig> configs = new HashMap<>();

    public static YamlConfig getConfig(String id) {
        if(id == null || id.isEmpty())
            return null;

        return configs.get(id);
    }

    public static void load() {
        loadRegions();
        loadAttractions();
        loadStatusData();
        AttractionMenu.load();
        loadDatabase();
    }

    private static void loadRegions() {
        YamlConfig config = ThemeParkPlugin.getInstance().getRegions();
        FileConfiguration fc = config.getConfig();
        if(!fc.contains("regions"))
            return;

        ConfigurationSection section = fc.getConfigurationSection("regions");
        if(section == null)
            return;

        for(String id : section.getKeys(false)) {
            if(API.isRegion(id))
                continue;

            String path = "regions."+id+".";
            String name = fc.getString(path+"name");
            List<String> lore = fc.getStringList(path+"lore");
            if(fc.contains(path+"material")) {
                Material material = Material.getMaterial(fc.getString(path + "material"));
                short data = Short.parseShort(fc.getString(path + "data"));
                Region region = new Region(id, name, lore, material, data);
                API.addRegion(region);
                continue;
            }

            Region region = new Region(id, name, lore);
            API.addRegion(region);
        }
    }

    public static YamlConfig addRegion(String id) {
        if(!API.isRegion(id))
            return null;

        File root = new File(ThemeParkPlugin.getInstance().getDataFolder(), "attractions");
        if(!root.exists())
            if(!root.mkdirs())
                return null;

        YamlConfig config = new YamlConfig("attractions/"+id, ThemeParkPlugin.getInstance());
        configs.put(id, config);
        return config;
    }

    private static void loadAttractions() {
        File root = new File(ThemeParkPlugin.getInstance().getDataFolder(), "attractions");
        if(!root.exists())
            if(!root.mkdirs())
                return;

        try {
            for(Region region : API.getRegions().values()) {
                YamlConfig config = new YamlConfig("attractions/"+region.getId(), ThemeParkPlugin.getInstance());
                configs.put(region.getId(), config);
                FileConfiguration fc = config.getConfig();
                if(fc == null)
                    continue;

                if(!fc.contains("attractions")) {
                    fc.set("attractions", new ArrayList<>());
                    Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), config::save);
                    continue;
                }

                ConfigurationSection section = fc.getConfigurationSection("attractions");
                if(section == null)
                    return;

                for(String id : section.getKeys(false)) {
                    if(API.isAttraction(id))
                        continue;

                    String path = "attractions."+id+".";
                    String name = fc.getString(path+"name");
                    Type type = Type.getType(fc.getString(path+"type"));
                    if(type == null)
                        continue;

                    Status status = Status.getStatus(fc.getString(path+"status"));
                    if(!type.containsStatus(status)) {
                        status = type.getDefault();
                        fc.set(path+"status", status.toString());
                    }

                    Location location = LocationSerializer.toLocation(fc.getString(path+".location"));
                    Attraction attraction = new Attraction(id, name, region.getId(), location, type, status);
                    API.addAttraction(attraction);
                }

                Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), config::save);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void loadStatusData() {
        YamlConfig config = ThemeParkPlugin.getInstance().getSettings();
        for(Status status : Status.values()) {
            String name = "UNKNOWN";
            if(config.getConfig().contains(status.toString() + ".name"))
                name = config.getConfig().getString(status.toString() + ".name");

            Material material = Material.STAINED_CLAY;
            if(config.getConfig().contains(status.toString() + ".material")) {
                material = Material.getMaterial(config.getConfig().getString(status.toString() + ".material"));
                if(material == null)
                    material = Material.STAINED_CLAY;
            }

            short data = 0;
            if(config.getConfig().contains(status.toString() + ".data")) {
                String string = config.getConfig().getString(status.toString() + ".data");
                if (!string.isEmpty())
                    data = Short.parseShort(string);
            }

            boolean teleport = false;
            if(config.getConfig().contains(status.toString() + ".teleport"))
                teleport = config.getConfig().getBoolean(status.toString() + ".teleport");

            StatusManager.addStatus(status, name, material, data, teleport);
        }
    }

    private static void loadDatabase() {
        DB db = DBManager.getDatabase("public");
        if(db == null)
            return;

        boolean b = false;
        StringBuilder builder = new StringBuilder();
        for(Region region : API.getRegions().values()) {
            if(b) {
                builder.append(",");
            } else {
                b = true;
            }

            builder.append("(?,?)");
        }


        int i = 1;
        HashMap<Integer, Object> obj = new HashMap<>();
        for(Region region : API.getRegions().values()) {
            obj.put(i++, region.getId());
            obj.put(i++, region.getName());
        }

        db.execute("INSERT INTO region(id, name) VALUES "+builder.toString()+" ON DUPLICATE KEY UPDATE name=VALUES(name)", obj);
        b = false;
        builder = new StringBuilder();
        for(Attraction attraction : API.getAttractions().values()) {
            if(b) {
                builder.append(",");
            } else {
                b = true;
            }

            builder.append("(?, ?, ?, ?, ?)");
        }

        i = 1;
        obj = new HashMap<>();
        for(Attraction attraction : API.getAttractions().values()) {
            obj.put(i++, attraction.getId());
            obj.put(i++, attraction.getName());
            obj.put(i++, attraction.getRegionId());
            obj.put(i++, attraction.getType().toString());
            obj.put(i++, attraction.getStatus().toString());
        }


        db.execute("INSERT INTO attraction(id, name, region_id, type, status) VALUES "+builder.toString()+" ON DUPLICATE KEY UPDATE name=VALUES(name), region_id=VALUES(region_id), type=VALUES(type), status=VALUES(status)", obj);
        b = false;
        builder = new StringBuilder();
        for(Status status : Status.values()) {
            if(b) {
                builder.append(",");
            } else {
                b = true;
            }

            builder.append("(?, ?)");
        }

        i = 1;
        obj = new HashMap<>();
        for(Status status : Status.values()) {
            obj.put(i++, status.toString());
            obj.put(i++, StatusManager.getName(status));
        }

        db.execute("INSERT INTO status(statusId, statusName) VALUES "+builder.toString()+" ON DUPLICATE KEY UPDATE statusName=VALUES(statusName)", obj);
    }

}
