package nl.iobyte.themepark.ridecount;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import nl.iobyte.menuapi.map.WeakConcurrentHashMap;
import nl.iobyte.themepark.ThemeParkPlugin;
import nl.iobyte.themepark.api.API;
import nl.iobyte.themepark.api.attraction.Attraction;
import nl.iobyte.themepark.api.ridecount.AttractionCount;
import nl.iobyte.themepark.api.ridecount.CountManager;
import nl.iobyte.themepark.database.DB;
import nl.iobyte.themepark.database.DBManager;
import nl.iobyte.workchain.components.Work;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

public class RideCountAPI {

    private static WeakConcurrentHashMap<String, String> data = new WeakConcurrentHashMap<>(5 * 60 * 1000);

    public static CompletableFuture<Integer> getCount(String name, Attraction attraction) {
        CompletableFuture<Integer> fc = new CompletableFuture<>();
        if(name == null || name.isEmpty() || attraction == null) {
            fc.complete(-1);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(-1);
            return fc;
        }

        getData(name).thenAccept(uuid -> {
            if(uuid == null) {
                fc.complete(-1);
                return;
            }

            getCount(uuid, attraction).thenAccept(fc::complete);
        });
        return fc;
    }

    public static CompletableFuture<Integer> getCount(UUID uuid, Attraction attraction) {
        CompletableFuture<Integer> fc = new CompletableFuture<>();
        if(uuid == null || attraction == null) {
            fc.complete(-1);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(-1);
            return fc;
        }

        AttractionCount count = CountManager.getCounter(attraction.getId());
        if(count != null && count.hasCount(uuid)) {
            fc.complete(count.getCount(uuid).getCount());
            return fc;
        }

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            Connection connection = db.getConnection();
            if(connection == null || connection.isClosed())
                return null;

            PreparedStatement statement = connection.prepareStatement("SELECT points FROM counts WHERE uuid=? AND attraction_id=?");
            statement.setString(1, uuid.toString());
            statement.setString(2, attraction.getId());
            statement.closeOnCompletion();
            return statement.executeQuery();
        }).abortIfNull(() -> fc.complete(-1)).nextTask(result -> {
            int i = -1;
            while(result.next()) {
                int points = result.getInt("points");
                if(points < 0)
                    continue;

                i = points;
            }

            result.close();
            return i;
        }).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    public static CompletableFuture<Boolean> addCount(String name, Attraction attraction, int amount) {
        CompletableFuture<Boolean> fc = new CompletableFuture<>();
        if(name == null || name.isEmpty() || attraction == null || amount < 0) {
            fc.complete(false);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(false);
            return fc;
        }

        getData(name).thenAccept(uuid -> {
            if(uuid == null) {
                fc.complete(false);
                return;
            }

            addCount(uuid, attraction, amount).thenAccept(fc::complete);
        });
        return fc;
    }

    public static CompletableFuture<Boolean> addCount(UUID uuid, Attraction attraction, int amount) {
        CompletableFuture<Boolean> fc = new CompletableFuture<>();
        if(uuid == null || attraction == null || amount < 0) {
            fc.complete(false);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(false);
            return fc;
        }

        AttractionCount count = CountManager.getCounter(attraction.getId());
        if(count != null && count.hasCount(uuid))
            Bukkit.getScheduler().runTask(ThemeParkPlugin.getInstance(), () -> count.addCount(uuid, amount));

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            Connection connection = db.getConnection();
            if(connection == null || connection.isClosed())
                return null;

            PreparedStatement statement = connection.prepareStatement("INSERT INTO counts(uuid, attraction_id, points) VALUES (?, ?, ?) ON CONFLICT(uuid, attraction_id) DO UPDATE SET points=points+?");
            statement.setString(1, uuid.toString());
            statement.setString(2, attraction.getId());
            statement.setInt(3, amount);
            statement.setInt(4, amount);
            statement.closeOnCompletion();
            return statement.executeUpdate() > 0;
        }).abortIfNull(() -> fc.complete(false)).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    private static CompletableFuture<UUID> getData(String name) {
        CompletableFuture<UUID> fc = new CompletableFuture<>();
        Bukkit.getScheduler().runTaskAsynchronously(ThemeParkPlugin.getInstance(), () -> {
            if(data.containsKey(name)) {
                fc.complete(UUID.fromString(data.get(name)));
                return;
            }

            Player player = Bukkit.getPlayerExact(name);
            if(player != null) {
                UUID uuid = player.getUniqueId();
                data.put(name, uuid.toString());
                fc.complete(uuid);
                return;
            }

            try {
                JsonObject object = sendGETRequestJSON("https://api.mojang.com/users/profiles/minecraft/" + name);
                if(object == null) {
                    data.put(name, null);
                    fc.complete(null);
                    return;
                }

                String id = object.get("id").getAsString();
                if(id == null || id.isEmpty()) {
                    data.put(name, null);
                    fc.complete(null);
                    return;
                }

                id = fixUUID(id);
                UUID uuid = UUID.fromString(id);
                data.put(name, id);
                fc.complete(uuid);
            } catch (Exception e) {
                data.put(name, null);
                fc.complete(null);
            }
        });

        return fc;
    }

    public static CompletableFuture<HashMap<UUID, Integer>> getTop(Attraction attraction, boolean week) {
        CompletableFuture<HashMap<UUID, Integer>> fc = new CompletableFuture<>();
        if(attraction == null) {
            fc.complete(null);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(null);
            return fc;
        }

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            Connection connection = db.getConnection();
            if(connection == null || connection.isClosed())
                return null;

            String timeSort = week ? "YEARWEEK(created_at, 1)=YEARWEEK(CURDATE(), 1)" : "DAYOFYEAR(created_at)=DAYOFYEAR(CURDATE())";
            PreparedStatement statement = connection.prepareStatement("SELECT uuid, SUM(points) AS count FROM ridecount WHERE attraction_id=? AND "+timeSort+" AND YEAR(created_at)=YEAR(CURDATE()) GROUP BY uuid ORDER BY points DESC LIMIT 3");
            statement.setString(1, attraction.getId());
            statement.closeOnCompletion();
            return statement.executeQuery();
        }).abortIfNull(() -> fc.complete(null)).nextTask(result -> {
            HashMap<UUID, Integer> top = new HashMap<>();
            while(result.next()) {
                UUID uuid = UUID.fromString(result.getString("uuid"));
                int points = result.getInt("points");
                if(points < 0)
                    continue;

                top.put(uuid, points);
            }

            result.close();
            return top;
        }).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    public static CompletableFuture<HashMap<UUID, Integer>> getTopTotal(Attraction attraction) {
        CompletableFuture<HashMap<UUID, Integer>> fc = new CompletableFuture<>();
        if(attraction == null) {
            fc.complete(null);
            return fc;
        }

        if(!API.isAttraction(attraction.getId())) {
            fc.complete(null);
            return fc;
        }

        Work.firstTask(() -> {
            DB db = DBManager.getDatabase("data");
            if(db == null)
                return null;

            Connection connection = db.getConnection();
            if(connection == null || connection.isClosed())
                return null;

            PreparedStatement statement = connection.prepareStatement("SELECT uuid, SUM(points) AS count FROM ridecount WHERE attraction_id=? GROUP BY uuid ORDER BY points DESC LIMIT 3");
            statement.setString(1, attraction.getId());
            statement.closeOnCompletion();
            return statement.executeQuery();
        }).abortIfNull(() -> fc.complete(null)).nextTask(result -> {
            HashMap<UUID, Integer> top = new HashMap<>();
            while(result.next()) {
                UUID uuid = UUID.fromString(result.getString("uuid"));
                int points = result.getInt("points");
                if(points < 0)
                    continue;

                top.put(uuid, points);
            }

            result.close();
            return top;
        }).lastTask(fc::complete).setFullExecute(true).execute();
        return fc;
    }

    private static String fixUUID(String uuid) {
        if(uuid == null || uuid.isEmpty())
            return null;

        StringBuilder sb = new StringBuilder(uuid);
        sb.insert(8, "-");
        sb = new StringBuilder(sb.toString());
        sb.insert(13, "-");
        sb = new StringBuilder(sb.toString());
        sb.insert(18, "-");
        sb = new StringBuilder(sb.toString());
        sb.insert(23, "-");

        return sb.toString();
    }

    private static JsonObject sendGETRequestJSON(String uri) throws IOException {
        URL url = new URL(uri);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();
        con.setRequestMethod("GET");
        con.setRequestProperty("User-Agent", "Mozilla/5.0");
        int code = con.getResponseCode();
        if (code == 404)
            return null;

        BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
        String inputLine;
        StringBuilder response = new StringBuilder();
        while ((inputLine = in.readLine()) != null)
            response.append(inputLine);

        in.close();
        JsonParser parser = new JsonParser();
        return parser.parse(response.toString()).getAsJsonObject();
    }

}
